#include <iostream>
#include <type_traits>

template<typename T>
void f(T&& arg)                             // akzeptiert R-Wert und nicht-const L-Wert
{                                           // keine Kopie
  if (std::is_reference_v<T>) {
    std::cout << "f<int&>(int&): " << arg  << '\n';
    arg = 8;                                // ändert Original
  }
  else {
    std::cout << "f<int>(int&&): " << arg  << '\n';
    arg = 9;                                // ändert Original, aber ohne Auswirkung
                                      // auf den Aufrufer, weil temporäres Objekt
  }
}

template<typename T>
void f(const T& arg)                        // akzeptiert R-Wert und L-Wert
{                                           // keine Kopie
    std::cout << "f<int>(const int&): " << arg  << '\n';

}

template<typename T>
void g(T&& arg)                              // akzeptiert L- und R-Wert
{                                            // keine Kopie
  if (std::is_reference_v<T>) {
    std::cout << "g<int&>(int&): " << arg  << '\n';
  }
  else {
    std::cout << "g<int>(int&&): " << arg  << '\n';
  }
}

template<typename T>
void h(const T& arg)                         // akzeptiert L- und R-Wert
{                                            // Kopie nur für R-Wert
    std::cout << "h<int>(const int&): " << arg  << '\n';
}

int main()
{
  int lwert {0};
  f(lwert);
  std::cout << "lwert geändert: " << lwert << '\n';
  f(2);                                      // geänderter Wert nicht zugreifbar
                                             // Das int-Objekt ist ein xvalue.
  const int ic {7};
  f(ic);                                     // Fehler, falls es f(const T& arg) nicht gibt
                                             // ic ist L-Wert, weil es einen Namen hat. 
  g(lwert);                                  // restliche Aufrufe: ok, keine Änderung des Arguments
  g(2);
  g(ic);
  h(lwert);
  h(3);
  h(ic);
}
