#ifndef THREADGROUP_H
#define THREADGROUP_H
#include <thread>
#include <vector>

class ThreadGroup {
public:
  ThreadGroup() = default;
  ~ThreadGroup() = default;       // Deklaration verhindert auch move-Operationen
  ThreadGroup(const ThreadGroup&) = delete;
  ThreadGroup& operator=(const ThreadGroup&) = delete;
  // Zur Dokumentation. Kopieren ist ohnehin nicht möglich, weil Threads
  // nicht kopiert werden können.

  template <typename F>
  void add_thread(F&& f)                     // thread(f) hinzufügen und starten
  {
     threads.emplace_back(f);
  }

  void join_all()                            // Auf Beendigung aller Threads warten
  {
    for (auto& t : threads) {
      t.join();
    }
  }

  [[nodiscard]] auto size() const
  { 
    return std::ssize(threads); 
  }
  // mögliche weitere Methoden weggelassen
private:
  std::vector<std::jthread> threads;
};
#endif

