#include "ThreadGroup.h"
#include <atomic>
#include <iostream>

// zu verarbeitende Struktur:
struct Daten {
  int a{0};
  int b{0};
  void incr()
  {
    ++a;
    ++b;
  }
};

namespace {
Daten nonAtomicDaten;
std::atomic<Daten> atomicDaten;
}

void testeAtomizitaet()
{
  constexpr int anzahl{10000};
  for (int i = 0; i < anzahl; ++i) {         // atomar
    auto temp = atomicDaten.load();          // Daten aus Memory nach temp kopieren
    temp.incr();                             // Werte erhöhen
    atomicDaten.store(temp);                 // temp zurückschreiben
    if (temp.a != temp.b) {
      std::cout << "atomicDaten: a und b sind ungleich! Ende.\n";
      break;                                 // Abbruch
    }
  }
  for (int i = 0; i < anzahl; ++i) {         // nicht-atomar
    nonAtomicDaten.incr();
    if (nonAtomicDaten.a != nonAtomicDaten.b) {
      std::cout << "nonAtomicDaten: a und b sind ungleich! Ende.\n";
      break;                                 // Abbruch
    }
  }
}

int main()
{
  ThreadGroup threadgroup;
  for (int i = 0; i < 10; ++i) {
    threadgroup.add_thread(testeAtomizitaet);
  }
  threadgroup.join_all();    // kann hier entfallen
}

