#ifndef GENERATOR_H
#define GENERATOR_H
// nach /cite{ISOC++} [dcl.fct.def.coroutine], ergänzt und modifiziert
#include <coroutine>

template<typename T>
struct Generator {
  struct promise_type;
  using handle = std::coroutine_handle<promise_type>;

  struct promise_type {
    T current_value;
    static auto get_return_object_on_allocation_failure() 
    {
      return Generator{nullptr}; 
    }
    auto get_return_object() { 
      return Generator{handle::from_promise(*this)}; 
    }
    auto initial_suspend() { return std::suspend_always{}; }
    auto final_suspend() noexcept { return std::suspend_always{}; }
    void unhandled_exception() { std::terminate(); }
    auto return_value(const T& value) {      // fuer co_return erforderlich
      return yield_value(value);
    }
    auto yield_value(const T& value) {
      current_value = value;
      return std::suspend_always{};
    }
    template<typename U>
    auto await_transform(U value)            // fuer co_await erforderlich
    {
      class awaiter
      {
        U value;
      public:
        explicit awaiter(U x) noexcept : value(x) {}
        bool await_ready() noexcept { return true; }
        void await_suspend(std::coroutine_handle<>) noexcept {}
        auto await_resume() noexcept { return value; }
      };
      return awaiter{ value };
    }
  };

  bool resume() {
    if(!cohandle) {
      return false;
    }
    cohandle.resume();
    return !cohandle.done();
  }

  T current_value() { return cohandle.promise().current_value; }
  Generator(Generator const&) = delete;
  Generator& operator=(Generator const&) = delete;
  Generator(Generator && rhs) : cohandle(rhs.cohandle) 
  { 
    rhs.cohandle = nullptr; 
  }
  ~Generator() { if (cohandle) cohandle.destroy(); }
private:
  Generator(handle h) : cohandle(h) {}
  handle cohandle;
};

template<>
struct Generator<void> {
  struct promise_type;
  using handle = std::coroutine_handle<promise_type>;

  struct promise_type {
    static auto get_return_object_on_allocation_failure()
    {
      return Generator{nullptr}; 
    }
    auto get_return_object() 
    {
      return Generator{handle::from_promise(*this)}; 
    }
    auto initial_suspend() { return std::suspend_always{}; }
    auto final_suspend() noexcept { return std::suspend_always{}; }
    void unhandled_exception() { std::terminate(); }
    void return_void() {}
  };

  bool resume() {
    if(!cohandle) {
      return false;
    }
    cohandle.resume();
    return !cohandle.done();
  }

  Generator(Generator const&) = delete;
  Generator(Generator && rhs) : cohandle(rhs.cohandle) { rhs.cohandle = nullptr; }
  ~Generator() { if (cohandle) cohandle.destroy(); }
private:
  Generator(handle h) : cohandle(h) {}
  handle cohandle;
};
#endif
