#include <fstream>
#include <iostream>
#include <iterator>
#include <string>
#include <vector>

// Funktions-Template zur Ermittlung der Kategorie des Iterators
template <class Iterator>
[[nodiscard]] auto getIteratortyp(const Iterator&)
{
  typename std::iterator_traits<Iterator>::iterator_category kategorie;
  return kategorie;
}

// überladene Funktionen
void welcherIterator(const std::input_iterator_tag&)
{ 
  std::cout << "Input-Iterator\n";
}

void welcherIterator(const std::output_iterator_tag&)
{ 
  std::cout << "Output-Iterator\n";
}

void welcherIterator(const std::forward_iterator_tag&)
{ 
  std::cout << "Forward-Iterator\n"; 
}

void welcherIterator(const std::random_access_iterator_tag&) 
{ 
  std::cout << "Random-Access-Iterator\n"; 
}

using namespace std;
int main()                                   // Anwendung
{
  // Bei Grunddatentypen (hier: ein Zeiger) wird das partiell spezialisierte
  // iterator_traits<T*>- Template von Seite 892 benutzt.
  int* ip = nullptr;                                   // Random-Access-Iterator
  // Anzeige des Iteratortyps
  welcherIterator(getIteratortyp(ip));
  welcherIterator(iterator_traits<int*>::iterator_category());

  // Definition eines Datei-Objekts zum Lesen (eine tatsächliche Datei
  // ist nicht erforderlich, es geht nur um den Typ)
  ifstream Source;

  istream_iterator<string> IPos(Source);  // Ein istream_iterator ist ein Input-Iterator.
  // Anzeige des Iteratortyps
  welcherIterator(getIteratortyp(IPos));     // oder:
  welcherIterator(
               iterator_traits<istream_iterator<string>>::iterator_category());

  ofstream ziel;                        // Definition eines Datei-Objekts zum Schreiben
  ostream_iterator<string> OPos(ziel);  // Ein ostream_iterator ist ein Output-Iterator.
  // Anzeige des Iteratortyps
  welcherIterator(getIteratortyp(OPos));     // oder:
  welcherIterator(
               iterator_traits<ostream_iterator<string>>::iterator_category());
  vector<int> v(10);
  // Anzeige des Iteratortyps
  welcherIterator(getIteratortyp(v.begin()));
  // oder ein anderer Iterator als begin()
  welcherIterator(iterator_traits<vector<int>::iterator>::iterator_category());
}

