#ifndef ARRAY2D_H
#define ARRAY2D_H
#include <iostream>
#include <stdexcept>
#include <string>
#include <vector>

template <typename T>
class [[nodiscard]] Array2d {
public:
  Array2d(int z, int s, const T& wert = T())
  : zeilen_{z}, spalten_{s}, daten{std::vector<T>(z * s, wert)}
  { }

  [[nodiscard]] auto zeilen() const { return zeilen_; }
  [[nodiscard]] auto spalten() const { return spalten_; }

  void check_indizes(int z, int s) const
  {
    if (z < 0 || z >= zeilen_) {
      throw std::range_error("Zeile " + std::to_string(z) + " ungültig!");
    }
    if (s < 0 || s >= spalten_) {
      throw std::range_error("Spalte" + std::to_string(s) + " ungültig!");
    }
  }
  // C++23
  const auto& operator[](int z, int s) const 
  { 
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[z * spalten_ + s]; 
  }
  auto& operator[](int z, int s) 
  {
#ifdef INDEXCHECK
    check_indizes(z, s);
#endif
    return daten[z * spalten_ + s]; 
  }
private:
  int zeilen_;
  int spalten_;
  std::vector<T> daten;
};

template <typename T>
void print(const Array2d<T>& a, std::ostream& ausgabe = std::cout) 
{
  for (int z = 0; z < a.zeilen(); ++z) {
    for (int s = 0; s < a.spalten(); ++s) {
      ausgabe << a[z, s] << " ";
    }
    ausgabe << '\n';
  }
  ausgabe << '\n';
}
#endif
