#include <algorithm>
#include <cassert>
#include <cmath>
#include <compare>
#include <iostream>
#include <vector>

class Punkt {
public:
  constexpr Punkt(double x, double y) : x_(x), y_(y) {}
  [[nodiscard]] constexpr double x() const { return x_;}
  [[nodiscard]] constexpr double y() const { return y_;}
  // unten selbst definierter <=> erfordert zusätzlich ==
  [[nodiscard]] friend constexpr bool operator==(const Punkt&, const Punkt&) = default;
private:
  double x_;
  double y_;
};

[[nodiscard]] constexpr auto operator<=>(const Punkt& lhs, const Punkt& rhs)
{
  auto abstand_lhs { std::sqrt(lhs.x()*lhs.x() + lhs.y()*lhs.y())};
  auto abstand_rhs { std::sqrt(rhs.x()*rhs.x() + rhs.y()*rhs.y())};
  if (abstand_lhs < abstand_rhs) {
    return std::weak_ordering::less;
  } else if (abstand_lhs > abstand_rhs) {
    return std::weak_ordering::greater;
  }
  return std::weak_ordering::equivalent;
}

int main()
{
  std::vector<Punkt> punkte { {1.2, 2.1}, {1.0, 0.1}, {1.0, 1.0}, 
                              {2.2, 2.2}, {2.0, 0.2}, {2.1, 1.2},
                              {0.2, 2.0}, {0.0, 0.0}, {0.1, 1.0}};
  std::sort(punkte.begin(), punkte.end());
  std::cout.setf(std::ios::fixed);
  std::cout.precision(1);
  for(auto p : punkte) {
    std::cout << '(' << p.x() << ", " << p.y() <<") "; 
  }
  std::cout << '\n';
  for(auto i {1z}; i < ssize(punkte); ++i) {
    if (punkte[i-1] <=> punkte[i] == 0) {
      std::cout << "Die Punkte " 
                << i-1 << ": (" 
                << punkte[i-1].x() << ", " << punkte[i-1].y() <<") und "
                << i << ": (" 
                << punkte[i].x() << ", " << punkte[i].y() 
                <<") sind äquivalent\n";
    }
  }
}
